﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

using System.IO;
using System.Windows.Media.Imaging;

namespace PaintSampleCS
{
    public partial class MainPage : UserControl
    {


        public MainPage()
        {
            InitializeComponent();
        }

        //保存ボタンをクリックしたときの処理です
        private void savePaint_Click(object sender, RoutedEventArgs e)
        {
            //SaveFileDialogを開きます
            SaveFileDialog saveDialog = new SaveFileDialog() { 
                DefaultExt="bmp",
                Filter = "ビットマップ(*.bmp)|*.bmp"
            };

            if (saveDialog.ShowDialog() == true)
            {
                //ファイルの保存するためのストリームを開きます
                using (Stream stream = saveDialog.OpenFile())
                {
                    //画像を取り込んだInkPresenterを対象にWriteableBitmapを生成します
                    WriteableBitmap wb = new WriteableBitmap(this.inkP, null);

                    //WriteableBitmapから出力用のバイト配列を取得します
                    byte[] buffer = GetBuffer(wb);

                    //Bitmapファイルとして保存します
                    stream.Write(buffer, 0, buffer.Length);
                }
            }
        }

        //WriteableBitmapから出力用のbyte配列を取得します
        private byte[] GetBuffer(WriteableBitmap bitmap)
        {
            int width = bitmap.PixelWidth;
            int height = bitmap.PixelHeight;

            MemoryStream ms = new MemoryStream();

            //BMP File Header(14 bytes)
            //the magic number(2 bytes):BM
            ms.WriteByte(0x42);
            ms.WriteByte(0x4D);

            //the size of the BMP file in bytes(4 bytes)
            int len = bitmap.Pixels.Length * 4 + 0x36;

            ms.WriteByte((byte)len);
            ms.WriteByte((byte)(len >> 8));
            ms.WriteByte((byte)(len >> 16));
            ms.WriteByte((byte)(len >> 24));

            //reserved(2 bytes)
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);

            //reserved(2 bytes)
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);

            //the offset(4 bytes)
            ms.WriteByte(0x36);
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);
            

            //Bitmap Information(40 bytes:Windows V3)
            //the size of this header(4 bytes)
            ms.WriteByte(0x28);
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);

            //the bitmap width in pixels(4 bytes)
            ms.WriteByte((byte)width);
            ms.WriteByte((byte)(width >> 8));
            ms.WriteByte((byte)(width >> 16));
            ms.WriteByte((byte)(width >> 24));

            //the bitmap height in pixels(4 bytes)
            ms.WriteByte((byte)height);
            ms.WriteByte((byte)(height >> 8));
            ms.WriteByte((byte)(height >> 16));
            ms.WriteByte((byte)(height >> 24));

            //the number of color planes(2 bytes)
            ms.WriteByte(0x01);
            ms.WriteByte(0x00);

            //the number of bits per pixel(2 bytes)
            ms.WriteByte(0x20);
            ms.WriteByte(0x00);

            //the compression method(4 bytes)
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);

            //the image size(4 bytes)
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);

            //the horizontal resolution of the image(4 bytes)
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);

            //the vertical resolution of the image(4 bytes)
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);

            //the number of colors in the color palette(4 bytes)
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);

            //the number of important colors(4 bytes)
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);
            ms.WriteByte(0x00);
            

            //Bitmap data
            for (int y = height - 1; y >= 0; y--)
            {
                for (int x = 0; x < width; x++)
                {
                    int pixel = bitmap.Pixels[width * y + x];

                    ms.WriteByte((byte)(pixel & 0xff)); //B
                    ms.WriteByte((byte)((pixel >> 8) & 0xff)); //G
                    ms.WriteByte((byte)((pixel >> 0x10) & 0xff)); //R
                    ms.WriteByte(0x00); //reserved
                }
            }
            

            return ms.GetBuffer();
        }

        //現在処理中のStrokeオブジェクトを保持します
        private System.Windows.Ink.Stroke currentStroke;

        //マウスが左クリックされたとき、InkPresenterに新しいStrokeを追加します
        private void inkP_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            this.inkP.CaptureMouse(); 
            currentStroke = new System.Windows.Ink.Stroke(); 
            currentStroke.StylusPoints.Add(e.StylusDevice.GetStylusPoints(inkP)); 
            currentStroke.DrawingAttributes.Color = Colors.Black;
            currentStroke.DrawingAttributes.Width = 5;
            currentStroke.DrawingAttributes.Height = 5;
            inkP.Strokes.Add(currentStroke);
        }

        //マウスが移動されたとき、Strokeに新しいStylusPointを追加します
        private void inkP_MouseMove(object sender, MouseEventArgs e)
        {
            if (currentStroke == null) return;
            currentStroke.StylusPoints.Add(e.StylusDevice.GetStylusPoints(inkP));
        }

        //マウスのキャプチャを失ったとき、現在のStrokeに対する処理を終了します
        private void inkP_LostMouseCapture(object sender, MouseEventArgs e)
        {
            currentStroke = null;
        }

        //画像を選択ボタンをクリックしたときの処理です
        private void imageSelect_Click(object sender, RoutedEventArgs e)
        {
            this.selectedImage.Text = null; 
            OpenFileDialog dialog = new OpenFileDialog(); 
            dialog.Multiselect = false; //単一選択    
            bool? dialogResult = dialog.ShowDialog();
            
            if (!dialogResult.HasValue || !dialogResult.Value)    
            {        
                return; //ファイルが選択されなかったときは何もしない    
            }    
            
            try    
            {
                //選択した画像ファイルを読み込み、       
                BitmapImage source = new BitmapImage();        
                source.SetSource(dialog.File.OpenRead());       
                //Imageのソースに設定        
                this.image.Source = source;         
                this.selectedImage.Text = dialog.File.Name;    
            }    
            catch    
            {        
                MessageBox.Show("画像が正しく読み込めませんでした");    
            }
        }
    }
}
